@@ -185,11 +185,10 @@
 
 
 (define copy-tree 
-  (let ((+documentation+ "(copy-tree lst) returns a full copy of lst")) ; but it does not handle cycles correctly
+  (let ((+documentation+ "(copy-tree lst) returns a full copy of lst"))
     (lambda (lis)
       (if (pair? lis)
-	  (cons (copy-tree (car lis))
-		(copy-tree (cdr lis)))
+	  (copy lis :readable)
 	  lis))))
 
 (define tree-member 
@@ -528,6 +527,8 @@ If func approves of one, index-if returns the index that gives that element's po
   (let ((+documentation+ "(collect-if type func sequence) gathers the elements of sequence that satisfy func, and returns them via type:\n\
     (collect-if list integer? #(1.4 2/3 1 1+i 2)) -> '(1 2)"))
     (lambda (type f sequence)
+      (unless (sequence? sequence)
+	(error 'wrong-type-arg "collect-if: sequence arg is ~A" sequence))
       (apply type (map (lambda (arg) (if (f arg) arg (values))) sequence)))))
 
 ;;; if type=list, this is slightly wasteful because list currently copies its args, so:
@@ -539,6 +540,8 @@ If func approves of one, index-if returns the index that gives that element's po
   (let ((+documentation+ "(remove-if type f sequence) returns via type the elements of sequence that do not satisfy func:\n\
     (remove-if list integer? #(1.4 2/3 1 1+i 2)) -> '(1.4 2/3 1+1i)"))
     (lambda (type f sequence)
+      (unless (sequence? sequence)
+	(error 'wrong-type-arg "remove-if: sequence arg is ~A" sequence))
       (collect-if type (lambda (obj) (not (f obj))) sequence))))
 
 (define nonce 
@@ -1002,7 +1005,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 	 ',class-name new-class                                  ; define the class as class-name in the calling environment
 	 
 	 ;; define class-name? type check
-	 (string->symbol (string-append (symbol->string ',class-name) "?"))
+	 (symbol (symbol->string ',class-name) "?")
 	 (lambda (obj)
 	   (and (let? obj)
 		(eq? (obj 'class-name) ',class-name))))
@@ -1010,7 +1013,7 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
        (varlet outer-env
 	 ;; define the make-instance function for this class.  
 	 ;;   Each slot is a keyword argument to the make function.
-	 (string->symbol (string-append "make-" (symbol->string ',class-name)))
+	 (symbol "make-" (symbol->string ',class-name))
 	 (apply lambda* (map (lambda (slot)
 			       (if (pair? slot)
 				   (list (car slot) (cdr slot))
